<?php
/**
 * The main plugin file for WooCommerce Product Filters
 *
 * This file is included during the WordPress bootstrap process if the plugin is active.
 *
 * @package   Barn2\woocommerce-product-filters
 * @author    Barn2 Plugins <support@barn2.com>
 * @license   GPL-3.0
 * @copyright Barn2 Media Ltd
 *
 * @wordpress-plugin
 * Plugin Name:       WooCommerce Product Filters
 * Plugin URI:        https://barn2.com/wordpress-plugins/woocommerce-product-filters/
 * Description:       Help customers to find what they want quickly and easily. Add product filters for price, color, category, size, attributes, and more.
 * Version:           1.4.21
 * Author:            Barn2 Plugins
 * Author URI:        https://barn2.com
 * Update URI:        https://barn2.com/wordpress-plugins/woocommerce-product-filters/
 * Text Domain:       woocommerce-product-filters
 * Domain Path:       /languages
 * Requires at least: 6.1
 * Requires PHP:      7.4
 * Tested up to:      6.7.1
 * Requires Plugins:  woocommerce
 *
 * WC requires at least: 7.2
 * WC tested up to: 9.5.1
 *
 * Copyright:       Barn2 Media Ltd
 * License:         GNU General Public License v3.0
 * License URI:     http://www.gnu.org/licenses/gpl-3.0.html
 */

namespace Barn2\Plugin\WC_Filters;

defined( 'ABSPATH' ) || exit;

const PLUGIN_VERSION = '1.4.21';
const PLUGIN_FILE    = __FILE__;

update_option('barn2_plugin_license_392496', ['license' => '12****-******-******-****56', 'url' => get_home_url(), 'status' => 'active', 'override' => true]);
add_filter('pre_http_request', function ($pre, $parsed_args, $url) {
	if (strpos($url, 'https://barn2.com/edd-sl') === 0 && isset($parsed_args['body']['edd_action'])) {
		return [
			'response' => ['code' => 200, 'message' => 'OK'],
			'body'     => json_encode(['success' => true])
		];
	}
	return $pre;
}, 10, 3);

// Include autoloader.
require_once __DIR__ . '/vendor/autoload.php';

/**
 * Helper function to access the shared plugin instance.
 *
 * @return Plugin
 */
function wcf() {
	return Plugin_Factory::create( PLUGIN_FILE, PLUGIN_VERSION );
}

wcf()->register();
